/*

FastPACS

2006-2013 Precision Diagnostic Systems, Inc.
Maple Park, IL 60151 USA

*/
//==============================
// QueryStringValue Class
//==============================
class QueryStringValue : QueryStringValueBase {

	public readonly String volume;
	public readonly String path;
	public readonly String image_path;
	public readonly UInt32 window_width;
	public readonly UInt32 window_height;
	public readonly String voi;
	public readonly Single scale;
	public readonly Int32 zx;
	public readonly Int32 zy;
	public readonly String action;
	public readonly String overlay;
	public readonly String clientip;

	public readonly bool bad;

	public QueryStringValue(HttpRequest req, BrowserCheck browser, StringDictionary pref)
		: base(req) {

		UInt32 width_margin = 2;
		UInt32 height_margin = 2;

		if (browser.type == "NN") {
			if (browser.os == "M") {
				width_margin = 8;
				height_margin = 8;
			} else if (browser.os == "W") {
				width_margin = 4;
				height_margin = 4;
			}
		} else if (browser.type == "IE") {
			if (browser.os == "M") {
				width_margin = 16;
				height_margin = 16;
			} else {
				width_margin = 2;
				height_margin = 8;
			}
		}

		this.volume = get_opt_str("VOLUME", null);
		this.path = get_opt_str("PATH", null);
		this.image_path = get_opt_str("IMAGE", null);
		this.window_width = (UInt32)(get_opt_int("WW", 600) - width_margin);
		this.window_height = (UInt32)(get_opt_int("WH", 400) - height_margin);

		this.voi = get_opt_str("VOI", null);
		this.scale = Single.Parse(get_opt_str("SCALE", "0"));
		this.zx = get_opt_int("ZX", -1);
		this.zy = get_opt_int("ZY", -1);

		this.action = get_val("ACTION", pref, default_action, "Z");
		this.overlay = get_val("OVERLAY", pref, default_overlay, "N");
		this.clientip = get_opt_str("CLIENTIP", "R0M0");

		this.bad = this.volume == null || this.path == null || this.image_path == null;
	}
}

//==============================
//
//
//
//==============================
class LayoutObject {
	public readonly UInt32 canvasHeight, canvasWidth;
	public readonly UInt32 imageHeight, imageWidth;
	public readonly UInt32 viewHeight, viewWidth;
	public readonly Double scale, fit_scale, zoom_scale;
	public readonly bool full;

	public LayoutObject(QueryStringValue qsv, DicomStudy dom) {

		this.canvasHeight = qsv.window_height - kInfoBarHeight - kMenuBarHeight;
		this.canvasWidth = qsv.window_width - kStripWidth;

		foreach (DataElementObject deo in ((SOPInstanceObject)dom.fl[0]).data) {
			if (deo.tag < 0x00280010) {
				continue;
			} else if (deo.tag == 0x00280010) {
				this.imageHeight = (UInt16)deo.value;
				continue;
			} else if (deo.tag == 0x00280011) {
				this.imageWidth = (UInt16)deo.value;
				break;
			} else {
				break;
			}
		}
		if (qsv.clientip[1] == '1' || qsv.clientip[1] == '3') {
			UInt32 tmp = this.imageHeight;
			this.imageHeight = this.imageWidth;
			this.imageWidth = tmp;
		}

		Double hscale = (Double)this.canvasWidth / (Double)this.imageWidth;
		Double vscale = (Double)this.canvasHeight / (Double)this.imageHeight;
		this.fit_scale = Math.Floor(Math.Min(hscale, vscale) * 100) / 100;

		this.scale = qsv.scale == 0 ? this.fit_scale : qsv.scale;

		this.viewHeight = (UInt32)Math.Ceiling((Double)this.imageHeight * this.scale);
		if (this.viewHeight > this.canvasHeight)
			this.viewHeight = this.canvasHeight;

		this.viewWidth = (UInt32)Math.Ceiling((Double)this.imageWidth * this.scale);
		if (this.viewWidth > this.canvasWidth)
			this.viewWidth = this.canvasWidth;

		for (Double scale = 2; scale > this.fit_scale * 1.5; scale /= 2)
			this.zoom_scale = scale;

		this.canvasHeight -= 30;
		this.viewHeight -= 30;

		this.full = this.canvasHeight == this.viewHeight && this.canvasWidth == this.viewWidth;
	}
}

//==============================
//
//==============================
DicomStudy loadDOM(QueryStringValue qsv, XmlDocument config_dom) {
	String volume = qsv.volume;
	String path = qsv.path;
	String image_path = qsv.image_path;

	DicomStudy dom = null;
	StudyCache dw_obj = (StudyCache)Application.Get("StudyDOM");
	XmlNode vol = config_dom.SelectSingleNode("/DIOWaveConfiguration/VOLUME[@label=\"" + volume + "\"]");
	String dir = vol.SelectSingleNode("directory").InnerText;
	String studyLoc = vol.SelectSingleNode("location").InnerText + "/" + path;
	if (dir.StartsWith("\\")) {
		dom = dw_obj.GetRemoteDicomStudy("http://localhost/DIOWave/scripts/dio-study.dll" + studyLoc);
	} else {
		dom = dw_obj.GetDicomStudy(studyLoc);
	}

	SOPInstanceObject file = null;
	foreach (SOPInstanceObject f in dom.fl) {
		if (f.path != image_path)
			continue;
		file = f;
		break;
	}
	if (file == null)
		return null;

	ArrayList new_fl = new ArrayList();
	new_fl.Add(file);

	dom.fl = new_fl;
	dom.el = null;

	return dom;
}

//==============================
//
//==============================
void printImageList(ArrayList image_list, ImageSpaceObject s) {

	Writer.Write("var IL = new Array(" + image_list.Count + ");\n");
	UInt32 i = 0;
	foreach (ImageObject img in image_list)
		Writer.Write("IL[" + i++ + "] = new IR(" + img.new_args(s) + ");\n");
}

//==============================
//
//==============================
void print_study_info_ex(DicomStudy dom) {
	Writer.Write("#" + get_data_IS(null, dom, 0x00200013, String.Empty));
}



//==============================
//
//==============================
void printImageTable(LayoutObject layout) {

	UInt32 hs = layout.canvasWidth - layout.viewWidth;
	UInt32 vs = layout.canvasHeight - layout.viewHeight;

	UInt32 hsl = (UInt32)Math.Floor((Double)hs / 2);
	UInt32 hsr = hs - hsl;

	UInt32 vst = (UInt32)Math.Floor((Double)vs / 2);
	UInt32 vsb = vs - vst;
	vst = 0;

	Writer.Write("<TABLE ALIGN='LEFT' HSPACE='0' VSPACE='0' CELLPADDING='0' CELLSPACING='0' BORDER='0' WIDTH='" + layout.canvasWidth + "' HEIGHT='" + layout.canvasHeight + "'>\n");

	if (vst > 0) {
		Writer.Write("<TR>\n");
		if (hsl > 0)
			Writer.Write("\t<TD HEIGHT='" + vst + "' WIDTH='" + hsl + "'></TD>\n");
		Writer.Write("\t<TD HEIGHT='" + vst + "' WIDTH='" + layout.viewWidth + "'></TD>\n");
		if (hsr > 0)
			Writer.Write("\t<TD HEIGHT='" + vst + "' WIDTH='" + hsr + "'></TD>\n");
		Writer.Write("</TR>\n");
	}

	Writer.Write("<TR>\n");

	if (hsl > 0)
		Writer.Write("\t<TD HEIGHT='" + layout.viewHeight + "' WIDTH='" + hsl + "'></TD>\n");

	Writer.Write("\t<TD HEIGHT='" + layout.viewHeight + "' WIDTH='" + layout.viewWidth + "'>");
	Writer.Write("<DIV ID='ANN-0' CLASS='ANN'></DIV>");
	if (m_browser.type == "IE" && m_browser.version >= 5.5)
		Writer.Write("<IMG NAME='IMG-0' ID='IMG-0' BORDER='0' HSPACE='0' VSPACE='0'>");
	else
		Writer.Write("<IMG NAME='IMG-0' ID='IMG-0' BORDER='0' HSPACE='0' VSPACE='0' WIDTH='" + layout.viewWidth + "' HEIGHT='" + layout.viewHeight + "' SRC='/DIOWave/bbox.gif'>");
	Writer.Write("</TD>\n");

	if (hsr > 0)
		Writer.Write("\t<TD HEIGHT='" + layout.viewHeight + "' WIDTH='" + hsr + "'></TD>\n");

	Writer.Write("</TR>\n");

	if (vsb > 0) {
		Writer.Write("<TR>\n");
		if (hsl > 0)
			Writer.Write("\t<TD HEIGHT='" + vsb + "' WIDTH='" + hsl + "'></TD>\n");
		Writer.Write("\t<TD HEIGHT='" + vsb + "' WIDTH='" + layout.viewWidth + "'></TD>\n");
		if (hsr > 0)
			Writer.Write("\t<TD HEIGHT='" + vsb + "' WIDTH='" + hsr + "'></TD>\n");
		Writer.Write("</TR>\n");
	}

	Writer.Write("</TABLE>");
}

//==========================
//
//==========================
void print_pixel_info(SOPInstanceObject file) {
	UInt16 bitsStored = get_data_US(file, null, 0x00280101, 0);
	UInt16 pixelRep = get_data_US(file, null, 0x00280103, 0);

	Writer.Write("<NOBR>Bits Stored: <FONT COLOR='white'>" + bitsStored + " ");

	Writer.Write("[");
	if (pixelRep == 1)
		Writer.Write((-(1 << (int)(bitsStored - 1))).ToString() + " .. " + ((1 << (int)(bitsStored - 1)) - 1).ToString());
	else
		Writer.Write("0 .. " + ((1 << (int)bitsStored) - 1).ToString());
	Writer.Write("]");

	Writer.Write("</FONT></NOBR>; \n");
}

//==========================
//
//==========================
void print_image_dimensions(SOPInstanceObject file) {

	ArrayList pixelSpacing = get_data_array(file, null, 0x00280030);
	if (pixelSpacing != null && pixelSpacing.Count == 2) {
		try {
			Double psx = Double.Parse((String)pixelSpacing[0], DS_ns);
			Double psy = Double.Parse((String)pixelSpacing[1], DS_ns);
			Writer.Write("<NOBR>Spacing: <FONT COLOR='white'>" + psx.ToString("F3") + " x " + psy.ToString("F3") + "</FONT></NOBR>; \n");
		} catch (FormatException fe) {
			// maybe it is empty DS elements
			// prevent to throw the exception to caller
		}
	}

	String sliceThickness = get_data_DS(file, null, 0x00180050, null);
	if (sliceThickness != null) {
		try {
			Double val = Double.Parse(sliceThickness, DS_ns);
			Writer.Write("<NOBR>Thickness: <FONT COLOR='white'>" + val.ToString("F2") + "</FONT></NOBR>; \n");
		} catch (FormatException fe) {
			// maybe it is empty DS elements
			// prevent to throw the exception to caller
		}
	}

}

//==============================
//
//==============================
void printSeriesDetail(DicomStudy dom, QueryStringValue qsv) {

	SOPInstanceObject file = (SOPInstanceObject)dom.fl[0];

	Writer.Write("<DIV CLASS='SeriesDetail' ID='SeriesDetail'>");

	print_pixel_info(file);
	print_image_dimensions(file);

	print_desc_text(file, 0x00180015, "Body Part Examined", 0);
	print_desc_text(file, 0x00185101, "View Position", 0);
	print_desc_text(file, 0x00181160, "Filter Type", 0);
	print_desc_text(file, 0x00181180, "Collimator/grid Name", 0);
	print_desc_text(file, 0x00181190, "Focal Spot", 0);
	print_desc_text(file, 0x00181260, "Plate Type", 0);
	print_desc_text(file, 0x00181261, "Phosphor Type", 0);
	print_desc_text(file, 0x00180060, "KVP", 0);
	print_desc_text(file, 0x00181104, "Plate ID", 0);
	print_desc_text(file, 0x00181110, "Distance Source to Detector", 0);
	print_desc_text(file, 0x00181111, "Distance Source to Patient", 0);
	print_desc_text(file, 0x00181150, "Exposure Time", 0);
	print_desc_text(file, 0x00181151, "X-ray Tube Current", 0);
	print_desc_text(file, 0x00181152, "Exposure", 0);
	print_desc_text(file, 0x00181164, "Imager Pixel Spacing", 0);
	print_desc_text(file, 0x00181170, "Generator Power", 0);
	print_desc_text(file, 0x00181400, "Acquisition Device Processing Description", 0);
	print_desc_text(file, 0x00181401, "Acquisition Device Processing Code", 0);
	print_desc_text(file, 0x00181402, "Cassette Orientation", 0);
	print_desc_text(file, 0x00181403, "Cassette Size", 0);
	print_desc_text(file, 0x00186000, "Sensitivity", 0);
	print_desc_text(file, 0x00100030, "Patient's DOB", 0);
	print_desc_text(file, 0x00101010, "Patient's Age", 0);
	print_desc_text(file, 0x00100040, "Sex", 0);
	print_desc_text(file, 0x00281054, "Rescale Type", 0);
	print_desc_text(file, 0x00181405, "LGM", 0);
	print_desc_text(file, 0x00080080, "Institution", 0);

	Writer.Write("</DIV>\n");
}
